﻿using SMP.Model.Finance;
using SMP.Model.TimeTask;
using XYY.Core.Standard.Data.Infrastructure;
using Dapper;
using SMP.Model.Enums;
using SMP.Model.Dto.Compute;
using Newtonsoft.Json;
using System.Text;

namespace SMP.Data.TimeTask
{
    public interface IFinance_TimeTaskRepository : IBaseRepository<Finance_TimeTask>
    {
        Task<bool> CreateTypeTaskByPort(IEnumerable<Logistisc_Order> orders);
        Task UpdateTaskStatus(FinanceEnum Enum, List<int> Ids);
        #region Finance_TimeTypeTaskDetail
        Task ClearTypeTaskDetail(int TaskId);
        Task<IEnumerable<Finance_TimeTypeTaskDetail>> GetTaskDetails(int taskId);
        Task<bool> CreateTypeTaskScopeDetail(Finance_TimeTask TaskInfo, List<ComputeAmountResult> computes);
        #endregion
    }

    public class Finance_TimeTaskRepository : BaseRepository<Finance_TimeTask>, IFinance_TimeTaskRepository
    {
        public Finance_TimeTaskRepository(IUnitOfWork unitOfWork) : base(unitOfWork)
        {

        }

        public async Task<bool> CreateTypeTaskByPort(IEnumerable<Logistisc_Order> orders)
        {
            bool result = true; int TaskId = 0;
            if (orders == null || orders.Count() == 0) { return result; }
            string LadingBillNumber = orders.FirstOrDefault().LadingBillNumber;
            var dbTaskInfo = _unitOfWork.QueryAsync<Finance_TimeTask>(x => x.LadingBillNumber == LadingBillNumber).Result.FirstOrDefault();
            if (dbTaskInfo != null)
            {
                dbTaskInfo.TaskStatus = FinanceEnum.WaitRead; TaskId = dbTaskInfo.Id;
                await _unitOfWork.UpdateAsync(dbTaskInfo);
            }
            else
            {
                Finance_TimeTask task = new Finance_TimeTask()
                {
                    StartTime = DateTime.Now,
                    EndTime = DateTime.Now,
                    OrderCount = orders.Count(),
                    LadingBillCount = 1,
                    LadingBillNumber = LadingBillNumber,
                    TaskType = TaskTypeEnum.DetailScope,
                    Remark = $"提单:[{LadingBillNumber}],存在口岸计费项，需进行末端运算，创建运算任务",
                };
                long dbId = await _unitOfWork.InsertAsync(task);
                TaskId = Convert.ToInt32(dbId);
            }
            await ClearTypeTaskDetail(TaskId);
            var typeTaskDetail = await OrderToTypeTaskDetail(TaskId, orders);
            await _unitOfWork.BulkToDBAsync<Finance_TimeTypeTaskDetail>(typeTaskDetail);
            return result;
        }
        public async Task UpdateTaskStatus(FinanceEnum Enum, List<int> Ids)
        {
            string sql = $"update Finance_TimeTask set TaskStatus={(int)Enum} where Id in ({string.Join(',', Ids)}) ";
            await _unitOfWork.ExecuteAsync(sql);
        }
        #region Finance_TimeTypeTaskDetail
        public async Task ClearTypeTaskDetail(int TaskId)
        {
            await _unitOfWork.ExecuteAsync($"delete Finance_TimeTypeTaskDetail where TaskId={TaskId}");
        }
        private async Task<List<Finance_TimeTypeTaskDetail>> OrderToTypeTaskDetail(int TaskId, IEnumerable<Logistisc_Order> orders)
        {
            List<Finance_TimeTypeTaskDetail> result = new List<Finance_TimeTypeTaskDetail>();
            foreach (var item in orders)
            {
                result.Add(new Finance_TimeTypeTaskDetail()
                {
                    TaskId = TaskId,
                    SystemNo = item.SystemNo,
                    LadingBillNumber = item.LadingBillNumber,
                    BoxNumber = item.BoxNumber,
                });
            }
            return result;
        }
        public async Task<IEnumerable<Finance_TimeTypeTaskDetail>> GetTaskDetails(int taskId)
        {
            return await _unitOfWork.QueryBySqlAsync<Finance_TimeTypeTaskDetail>($"select *from Finance_TimeTypeTaskDetail(nolock) where TaskId={taskId}");
        }
        private async Task ClearTypeTaskDetail(int TaskId, List<string> SystemNo)
        {
            string sql = $"delete Finance_TimeTypeTaskDetail where TaskId={TaskId} and SystemNo in @SystemNo";
            await _unitOfWork.ExecuteAsync(sql, new { SystemNo = SystemNo });
        }
        /// <summary>
        /// 末端预估错误数据记录
        /// 预估执行会分批，每次最多1K
        /// </summary>
        /// <param name="TaskId"></param>
        /// <param name="computes"></param>
        /// <returns></returns>
        public async Task<bool> CreateTypeTaskScopeDetail(Finance_TimeTask TaskInfo, List<ComputeAmountResult> computes)
        {
            if (computes == null || computes.Count == 0) { return true; }
            List<string> systemNo = computes.Select(x => x.BillNumber).Distinct().ToList();
            if (string.IsNullOrEmpty(TaskInfo.LadingBillNumber))//时间拉取任务
            {
                List<Finance_TimeTypeTaskDetail> result = new List<Finance_TimeTypeTaskDetail>();
                await ClearTypeTaskDetail(TaskInfo.Id, systemNo);
                foreach (var item in systemNo)
                {
                    result.Add(new Finance_TimeTypeTaskDetail()
                    {
                        TaskId = TaskInfo.Id,
                        SystemNo = item,
                        ErrorDetailJson = JsonConvert.SerializeObject(computes.Where(x => x.BillNumber == item)),
                    });
                }
                return await _unitOfWork.BulkToDBAsync<Finance_TimeTypeTaskDetail>(result);
            }
            else//口岸末端重算任务
            {
                StringBuilder sb = new StringBuilder();
                foreach (var item in systemNo)
                {
                    sb.Append($" update Finance_TimeTypeTaskDetail set ErrorDetailJson='{JsonConvert.SerializeObject(computes.Where(x=>x.BillNumber==item))}' where TaskId={TaskInfo.Id} and SystemNo='{item}' ");
                }
                await _unitOfWork.ExecuteAsync(sb.ToString());
                return true;
            }
            
        }
        #endregion
    }
}
