﻿using Microsoft.Extensions.Caching.Distributed;
using SMP.Model.Dto.Compute;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SMP.Model.Compute;
using NCalc;
using System.IO.Compression;
using System.Collections.Concurrent;
using SMP.Model.Enums;
using SMP.Common.Unit;
using XYY.Core.Standard.Mvc;

namespace SMP.Service.Compute
{
    public interface IComputeAmountService
    {
        Task<List<ComputeAmountResult>> MultipleCompute(List<ComputeAmountSouce> souces);
    }

    public class ComputeAmountService : IComputeAmountService
    {
        public IDistributedCache _cache;
        public IComputeExpenseItemService ItemService;
        public ISysRateService RateService;

        public ComputeAmountService(IDistributedCache cache, IComputeExpenseItemService itemService, ISysRateService rateService)
        {
            _cache = cache;
            ItemService = itemService;
            RateService = rateService;
        }

        public async Task<List<ComputeAmountResult>> MultipleCompute(List<ComputeAmountSouce> souces)
        {
            //每次调用方法，自身做一次缓存处理，防止取缓存与解压缩所带来的性能丢失
            ConcurrentDictionary<int, Compute_ExpenseItem> dict = new ConcurrentDictionary<int, Compute_ExpenseItem>();
            List<ComputeAmountResult> results = new List<ComputeAmountResult>();
            foreach (var item in souces)
            {
                Compute_ExpenseItem expresnse = null;
                if (!dict.TryGetValue(item.ExpenseItemId, out expresnse))
                {
                    expresnse = await GetExpense(item.ExpenseItemId);
                    dict.TryAdd(item.ExpenseItemId, expresnse);
                }
                if (expresnse != null && expresnse.ItemDetails != null)
                {
                    foreach (var detail in expresnse.ItemDetails)
                    {
                        ComputeAmountResult result = new ComputeAmountResult
                        {
                            BillNumber = item.BillNumber,
                            ExpenseId = detail.ExpenseId,
                            ExpenseItemId = item.ExpenseItemId
                        };

                        Compute_Expense version = null;
                        if (item.BindBillId > 0)
                        {
                            //有绑定过提单数据
                            var versionList = detail.DetailVersions.Where(x => x.BindBillId == item.BindBillId || x.VersionDate <= item.ComputeDate);
                            if (versionList.Count() > 0)
                            {
                                if (versionList.Any(x => x.BindBillId == item.BindBillId))
                                {
                                    //有绑定提单数据取提单数据
                                    version = versionList.First(x => x.BindBillId == item.BindBillId);
                                }
                                else
                                {
                                    ///没有取第一个版本
                                    version = versionList.First();
                                }
                            }
                        }
                        else
                        {
                            //非绑定提单数据
                            version = detail.DetailVersions.Where(x => x.VersionDate <= item.ComputeDate && x.BindBillId == 0).FirstOrDefault();
                        }
                        if (version != null && version.Details != null)
                        {
                            result.ExpenseName = version.Name;
                            var rate = await RateService.QueryRate(item.ComputeDate, version.CurrenyId);
                            result.BillCurrency = rate.Currency;
                            result.SettlementRate = rate.ToCNY;
                            ///重量处理
                            if (item.Weight <= version.Min)
                            {
                                item.Weight = version.Min;
                            }
                            //if (version.Stepping > 0)
                            //    item.Weight = Math.Ceiling(item.Weight / version.Stepping) * version.Stepping;
                            //

                            //取运费
                            var detailItem =
                                version.Details.FirstOrDefault(x =>
                                (item.Weight <= x.ToG && x.WeightType == Model.Enums.WeightType.运费 && x.WeightUnit != WeightUnit.M3) ||
                                (item.M3 <= x.Weight && x.WeightType == WeightType.运费 && x.WeightUnit == WeightUnit.M3)
                                );
                            if (detailItem != null)
                            {
                                //取操作费
                                var opDetailItem =
                                    version.Details.FirstOrDefault(x =>
                                    ((item.Weight <= x.ToG && x.WeightUnit != WeightUnit.M3) ||
                                (item.M3 <= x.Weight && x.WeightUnit == WeightUnit.M3))
                                    && x.WeightType == Model.Enums.WeightType.操作费
                                    && x.FormulaId == detailItem.FormulaId);
                                if (version.Zones != null)
                                {
                                    List<Compute_Zone> mathZones = new List<Compute_Zone>();
                                    foreach (var zone in version.Zones)
                                    {
                                        bool isMath = false;
                                        List<bool> AllMath = new List<bool>();
                                        foreach (var method in zone.ZoneMethods)
                                        {
                                            bool methodMath = false;
                                            if (method.Mode == Model.Enums.ZoneMode.邮编明细列表 && item.ZipCode != null)
                                            {
                                                //去空格，去无效字符
                                                item.ZipCode = item.ZipCode.Trim().Replace(" ", "");
                                                string[] mathZips = method.OriginalContent.Replace("\r\n", "\r").Split('\r', '\n', ',', ';');
                                                int zipCodeLength = mathZips.Max(x => x.Length);
                                                if (item.ZipCode.Length > zipCodeLength)
                                                {
                                                    //如果是邮编明细的话，邮编长度必须固定
                                                    //邮编从左开始截取
                                                    item.ZipCode = item.ZipCode.Substring(0, zipCodeLength);
                                                }
                                                methodMath = mathZips.Contains(item.ZipCode);
                                            }
                                            else if (method.Mode == Model.Enums.ZoneMode.邮编起止列表 && item.ZipCode != null)
                                            {
                                                //去空格，去无效字符
                                                item.ZipCode = item.ZipCode.Trim().Replace(" ", "");
                                                //获取组
                                                string[] mathZips = method.OriginalContent.Replace("\r\n", "\r").Split('\r', '\n', ',', ';');
                                                foreach (var mathZip in mathZips)
                                                {
                                                    //zip3
                                                    if (mathZip.Contains('-'))
                                                    {
                                                        var m2 = mathZip.Split('-');
                                                        int zipCodeLenght = Math.Max(m2[0].Length, m2[1].Length);
                                                        item.ZipCode = item.ZipCode.Substring(0, zipCodeLenght);
                                                        if (int.TryParse(m2[0], out var start)
                                                            && int.TryParse(m2[1], out var end)
                                                            && int.TryParse(item.ZipCode, out var zip))
                                                        {
                                                            isMath = zip >= start && zip <= end;
                                                        }
                                                    }
                                                }
                                            }
                                            else if (method.Mode == Model.Enums.ZoneMode.地址正则 && item.Stree != null)
                                            {
                                                if (method.OriginalContent == ".*")
                                                {
                                                    methodMath = true;
                                                }
                                                else
                                                {

                                                    item.Stree = item.Stree.Trim();
                                                    System.Text.RegularExpressions.Regex regex
                                                        = new System.Text.RegularExpressions.Regex(method.OriginalContent);

                                                    methodMath = regex.IsMatch(item.Stree);
                                                }
                                            }
                                            else if (method.Mode == Model.Enums.ZoneMode.国家列表 && item.CountryCode != null)
                                            {
                                                item.CountryCode = item.CountryCode.Trim();
                                                string[] mathCoutryCodes = method.OriginalContent.Replace("\r\n", "\r").Split('\r', '\n', ',', ';');
                                                methodMath = mathCoutryCodes.Contains(item.CountryCode);

                                            }
                                            else if (method.Mode == Model.Enums.ZoneMode.跟踪号正则 && item.TrackingNumber != null)
                                            {
                                                item.TrackingNumber = item.TrackingNumber.Trim();
                                                System.Text.RegularExpressions.Regex regex =
                                                    new System.Text.RegularExpressions.Regex(method.OriginalContent);
                                                methodMath = regex.IsMatch(item.TrackingNumber);
                                            }
                                            else if (method.Mode == Model.Enums.ZoneMode.API接口)
                                            {
                                                try
                                                {

                                                    string url = "http://localhost:7130/RemoteService";
                                                    var req = new
                                                    {
                                                        ApiName = method.OriginalContent,
                                                        ReceiveName = string.Empty,
                                                        ReceiveCountryCode = item.CountryCode,
                                                        ReceiveState = item.State,
                                                        ReceiveCity = item.City,
                                                        ReceiveZipCode = item.ZipCode,
                                                        ReceiveStree = item.Stree
                                                    };
                                                    string str = Newtonsoft.Json.JsonConvert.SerializeObject(req);
                                                    HttpContent content = new StringContent(str);
                                                    content.Headers.ContentType = new System.Net.Http.Headers.MediaTypeHeaderValue("application/json");
                                                    System.Net.Http.HttpClient client = new HttpClient();

                                                    var response = await client.PostAsync(url, content);
                                                    response.EnsureSuccessStatusCode();//用来抛异常的
                                                    string responseBody = await response.Content.ReadAsStringAsync();
                                                    var r = Newtonsoft.Json.JsonConvert.DeserializeObject<ApiJsonModel<bool>>(responseBody);
                                                    methodMath = r.data;
                                                }
                                                catch (Exception ex)
                                                {
                                                    Console.Write(ex.Message);
                                                    methodMath = false;
                                                }


                                            }
                                            else if (method.Mode == Model.Enums.ZoneMode.提单落地口岸 && item.Port != null)
                                            {
                                                item.Port = item.Port.Trim();
                                                methodMath = item.Port == method.OriginalContent.Trim();
                                            }
                                            else
                                            {
                                                methodMath = false;
                                            }
                                            AllMath.Add(methodMath);
                                        }
                                        //不包含匹配失败的规则
                                        isMath = !AllMath.Any(x => !x);
                                        if (isMath)
                                        {
                                            mathZones.Add(zone);
                                        }
                                    }

                                    if (mathZones.Count == 0)
                                    {
                                        result.Message = $"取明细价格失败，分区未匹配，请检查订单信息与相关费用[ExpenseItemId]{item.ExpenseItemId}设置";
                                    }
                                    else if (mathZones.Count > 1)
                                    {
                                        result.Message = $"取明细价格失败，分区存在匹配冲突{string.Join(",", mathZones.Select(x => x.Name).ToArray()) }" +
                                            $"，请检查订单信息与相关费用[ExpenseItemId]{item.ExpenseItemId}设置";
                                    }
                                    else
                                    {
                                        result.IsSuccess = true;
                                        result.ExpenseVersion = version.VersionDate.Value;

                                        decimal fee = detailItem.ZoneValue.Where(x => x.ZoneId == mathZones[0].Id).FirstOrDefault()?.Fee ?? 0;
                                        result.Amount = fee;
                                        result.Currency = "CNY";
                                        result.Zones = String.Join(";", mathZones[0].Name);
                                        ReplaterFormula(detailItem, "运费", fee.ToString());
                                        if (version.ComputeType == Model.Enums.ComputeType.运费加操作费)
                                        {
                                            if (opDetailItem == null)
                                            {
                                                result.Message = $"当前计费模式需要操作费，当前重量找不到匹配的操作费";
                                                continue;
                                            }
                                            decimal operFee = opDetailItem?.ZoneValue?.Where(x => x.ZoneId == mathZones[0].Id).FirstOrDefault()?.Fee ?? 0;
                                            ReplaterFormula(detailItem, "操作费", operFee.ToString());

                                        }
                                        //是否(最大值(体重,称重)>首重,首重价格+向上取整((最大值(称重,体重)-首重)÷续重)×续重价格,首重价格)

                                        decimal toWeight = item.Weight;


                                        if (detailItem.WeightUnit == WeightUnit.OZ)
                                        {
                                            toWeight = item.Weight / ConvertWeightHelper.OZToG;
                                        }
                                        else if (detailItem.WeightUnit == WeightUnit.LBS)
                                        {
                                            toWeight = item.Weight / ConvertWeightHelper.LBSToG;
                                        }
                                        else if (detailItem.WeightUnit == WeightUnit.KG)
                                        {
                                            toWeight = item.Weight / ConvertWeightHelper.KGToG;
                                        }
                                        else if (detailItem.WeightUnit == WeightUnit.M3)
                                        {
                                            toWeight = item.Weight / ConvertWeightHelper.KGToG;
                                        }

                                        ReplaterFormula(detailItem, "票数", item.OrderQty.ToString());
                                        ReplaterFormula(detailItem, "称重", toWeight.ToString());
                                        ReplaterFormula(detailItem, "立方米", item.M3.ToString());
                                        ReplaterFormula(detailItem, "体积", item.Volume.ToString());
                                        ReplaterFormula(detailItem, "步进", version.Stepping.ToString());
                                        ReplaterFormula(detailItem, "取整", "Ceiling");
                                        ReplaterFormula(detailItem, "四舍五入", "Round");
                                        ReplaterFormula(detailItem, "是否", "if");
                                        ReplaterFormula(detailItem, "取大值", "Max");
                                        ReplaterFormula(detailItem, "最大值", "Max");
                                        ReplaterFormula(detailItem, "申报价值", item.GoodsDeclareValue.ToString());
                                        ReplaterFormula(detailItem, "÷", "/");
                                        ReplaterFormula(detailItem, "×", "*");
                                        ReplaterFormula(detailItem, "+", "+");
                                        ReplaterFormula(detailItem, "≥", ">=");
                                        ReplaterFormula(detailItem, "≤", "<=");
                                        ReplaterFormula(detailItem, "。", ".");

                                        ReplaterFormula(detailItem, "０", "0");
                                        ReplaterFormula(detailItem, "３", "3");
                                        ReplaterFormula(detailItem, "２", "2");
                                        ReplaterFormula(detailItem, "１", "1");
                                        ReplaterFormula(detailItem, "６", "6");
                                        ReplaterFormula(detailItem, "５", "5");
                                        ReplaterFormula(detailItem, "４", "4");
                                        ReplaterFormula(detailItem, "９", "9");
                                        ReplaterFormula(detailItem, "８", "8");
                                        ReplaterFormula(detailItem, "７", "7");
                                        for (int i = 0; i < results.Count; i++)
                                        {
                                            ReplaterFormula(detailItem, $"第{i}步", results[i].Amount.ToString());
                                        }
                                        if (results.Count > 0)
                                        {
                                            ReplaterFormula(detailItem, $"上一步", results[results.Count - 1].Amount.ToString());
                                        }
                                        Expression expression = new Expression(detailItem.Formula.Formula.ToString());
                                        try
                                        {
                                            result.AmountinOriginalCurrency = (decimal)(double)expression.Evaluate();
                                        }
                                        catch (Exception ex)
                                        {
                                            result.Message = $"公式执行失败{detailItem.Formula.Formula}";
                                            result.IsSuccess = false;
                                        }
                                        result.Amount = Math.Round(result.AmountinOriginalCurrency * rate.ToCNY, 2, MidpointRounding.AwayFromZero);
                                        result.VersionId = version.Id;
                                        result.DetailId = version.Id;
                                        result.ExpenseName = version.Name;
                                    }
                                }
                                else
                                {
                                    result.Message = "按重量查找，未找到相关的有效计费版本";
                                }
                            }
                            else
                            {
                                result.Message = "按重量查找，未找到相关的有效计费区间";
                            }
                        }
                        else
                        {
                            result.Message = $"按时间({item.ComputeDate.ToString_yyyyMMdd()})查找，未找到相关的有效计费版本";
                        }
                        results.Add(result);
                    }
                }
            }
            return results;
        }

        private static void ReplaterFormula(Compute_ExpenseDetail? detailItem, string name, string fee)
        {
            detailItem.Formula.Formula = detailItem.Formula.Formula.Replace(name, fee);
        }

        public string @computeItemKey => "expense_";
        private async Task<Compute_ExpenseItem> GetExpense(int ComputeItemId)
        {
            string key = @computeItemKey + ComputeItemId;
            var cacheV = _cache.Get(key);
            if (cacheV == null)
            {
                //缓存压缩
                var expenseItem = await ItemService.GetExpenseItem(ComputeItemId);
                string josnValue = Newtonsoft.Json.JsonConvert.SerializeObject(expenseItem);
                var bytes = System.Text.UTF8Encoding.UTF8.GetBytes(josnValue);
                using var originalFileStream = new MemoryStream(bytes);
                using var compressedFileStream = new MemoryStream();
                using BrotliStream compressor = new BrotliStream(compressedFileStream, CompressionLevel.Fastest);
                originalFileStream.CopyTo(compressor);

                var compressedFileArray = compressedFileStream.ToArray();

                _cache.Set(key, compressedFileArray);
                return expenseItem;
            }
            else
            {
                //缓存解压
                using var originalFileStream = new MemoryStream(cacheV);
                using var outputFileStream = new MemoryStream();
                using var decompressor = new BrotliStream(originalFileStream, CompressionMode.Decompress);
                decompressor.CopyTo(outputFileStream);

                var decompressorArray = outputFileStream.ToArray();
                string orijson = System.Text.UTF8Encoding.UTF8.GetString(decompressorArray);
                if (!orijson.EndsWith("}"))
                    orijson = orijson + "}";
                return Newtonsoft.Json.JsonConvert.DeserializeObject<Compute_ExpenseItem>(orijson);
            }
        }
    }
}
