﻿using SMP.Data.Finance;
using SMP.Model.Dto.DataInterconnection;
using SMP.Model.Dto.Finance;
using SMP.Model.Enums;
using SMP.Model.Finance;
using SMP.Service.ThirdPartyRequest;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;

namespace SMP.Service.Finance
{
    #region Iservices
    public interface IFinance_Logistics_LadingBillService
    {
        #region 帐单
        Task<PageResult<Finance_Logistics_LadingBill>> QueryPage(QueryModel query);
        Task<long> Create(LadingBillDto dto);
        Task Edit(LadingBillDto dto);
        Task<long> ScopeBillCreate(int OrderQuerySettingId);
        Task ScopeBillEdit(LadingBillDto dto);
        Task RemoveLadingBill(int LadingBillId);
        Task RemoveScopeBill(int LadingBillId);
        Task<long> CustomerLadingBillCreate(LadingBillDto dto);
        Task CustomerLadingBillEdit(LadingBillDto dto);
        Task RemoveCustomerLadingBill(int LadingBillId);
        Task<Finance_Logistics_LadingBill> InsBill(int Id);
        #endregion

        #region 帐单费项
        Task<IEnumerable<Finance_Incidental>> GetIncidentals(int LadingBillId);
        #endregion

    }
    #endregion
    #region services 
    public class Finance_Logistics_LadingBillService : IFinance_Logistics_LadingBillService
    {
        IFinance_Logistics_LadingBillRepository _repository;
        private readonly IWMSDataInterconnectionService _wMSDataInterconnection;
        private readonly IOrderCostService _orderCostService;
        public Finance_Logistics_LadingBillService(IFinance_Logistics_LadingBillRepository repository, IWMSDataInterconnectionService wMSDataInterconnection, IOrderCostService orderCostService)
        {
            _repository = repository;
            this._wMSDataInterconnection = wMSDataInterconnection;
            this._orderCostService = orderCostService;
        }

        public async Task<PageResult<Finance_Logistics_LadingBill>> QueryPage(QueryModel query)
        {
            LadingBillDto returnResult = new LadingBillDto();
            if (query.QueryParamer == null) { query.QueryParamer = new List<QueryParamer>(); }
            query.QueryParamer.Add(new QueryParamer() { Filed = "IsDelete", Value = "0", Method = "Eq" });

            var result = await _repository.QueryPager2Async(query);

            return result;
        }
        public async Task<Finance_Logistics_LadingBill> InsBill(int Id)
        {
            var result = await _repository.QueryAsync(x=>x.Id==Id&&x.IsDelete==false);
            return result.FirstOrDefault();
        }
        #region 自飞头程帐单增改删

        public async Task<long> Create(LadingBillDto dto)
        {
            if (await _repository.IsExistsAsync(x => x.LadingBillNumber == dto.LadingBillNumber))
            {
                throw new Exception("已存在相同类型帐单" + dto.LadingBillNumber + "不允许创建新的同类型帐单");
            }
            ThirdPartyData_Input input = new ThirdPartyData_Input() { LadingBillNumber = dto.LadingBillNumber };
            var ThirdPartyLadingBillDto = await _wMSDataInterconnection.PullOrderInfo(input);
            if (ThirdPartyLadingBillDto == null || ThirdPartyLadingBillDto?.orders?.FirstOrDefault()?.LadingBillNumber == null) { throw new Exception("提单数据不存在，请确认提单号是否正确！"); }
            await _orderCostService.SyncOrderInfo(ThirdPartyLadingBillDto.orders);
            int BoxQty = ThirdPartyLadingBillDto.orders.Select(x => !string.IsNullOrEmpty(x.BoxNumber)).Distinct().Count();
            dto.BoxQty = BoxQty;
            dto.Pieces = ThirdPartyLadingBillDto.orders.Count();
            dto.ChargeWeight = dto.Weight = ThirdPartyLadingBillDto.TotalWeight ?? 0;
            var db = dto.Copy<Finance_Logistics_LadingBill>();
            int Id = (int)await _repository.InsertAsync(db);
            if (dto.SpreadlogicId != null && dto.SpreadlogicId > 0)
            {
                IncidentalsCompute compute = new IncidentalsCompute() { LadingBillId = Id, LadingBillNumber = dto.LadingBillNumber, IncidentalFees = dto.finances, Weight = dto.ServicesWeight ?? dto.Weight, Pieces = dto.Pieces ?? 0, orders = ThirdPartyLadingBillDto.orders, BillTime = dto.BillTime, Currency = dto.BillCurrency, _Destination = dto._Destination, M3 = (dto.M3 ?? 0.0M) };
                var tuple = await _orderCostService.AddIncidentals(compute);
                await _repository.UpdateByPrimaryKeyAsync(Id, "Cost", tuple.Item1);
                //计算订单均摊成本
                await _orderCostService.ApportionCompute(dto.SpreadlogicId ?? 0,"",
                    dto.LadingBillNumber, ThirdPartyLadingBillDto.orders, tuple.Item2, Model.Enums.SystemBillTypeEnum.LadingBill);
            }
            return Id;
        }

        public async Task Edit(LadingBillDto dto)
        {
            if (dto.Prepaid > 0) { throw new Exception("帐单已付款不可修改！"); }
            var oldDb = await _repository.GetAsync(dto.Id);
            IEnumerable<Logistisc_Order> orders = null;
            if (oldDb.LadingBillNumber != dto.LadingBillNumber)
            {
                if (await _repository.IsExistsAsync(x => x.LadingBillNumber == dto.LadingBillNumber && x.Id != dto.Id))
                {
                    throw new Exception("提单号重复，请确认填写是否正确！");
                }
                ThirdPartyData_Input input = new ThirdPartyData_Input() { LadingBillNumber = dto.LadingBillNumber };
                var ThirdPartyLadingBillDto = await _wMSDataInterconnection.PullOrderInfo(input);
                if (ThirdPartyLadingBillDto == null || ThirdPartyLadingBillDto?.orders?.FirstOrDefault()?.LadingBillNumber == null) { throw new Exception("提单数据不存在，请确认提单号是否正确！"); }
                orders = ThirdPartyLadingBillDto.orders;
                await _orderCostService.SyncOrderInfo(ThirdPartyLadingBillDto.orders);
            }
            else
            {
                orders = await _orderCostService.GetOrdersByLadingBillNumber(dto.LadingBillNumber);
            }
            await _repository.ClearIncidentals(dto.Id);
            var db = dto.Copy<Finance_Logistics_LadingBill>();
            IncidentalsCompute compute = new IncidentalsCompute() { LadingBillId = dto.Id, LadingBillNumber = dto.LadingBillNumber, IncidentalFees = dto.finances, Weight = dto.ServicesWeight ?? dto.Weight, Pieces = dto.Pieces ?? 0, orders = orders, BillTime = dto.BillTime, Currency = dto.BillCurrency, _Destination = dto._Destination, M3 = (dto.M3 ?? 0.0M) };
            var tuple = await _orderCostService.AddIncidentals(compute);
            db.Cost = tuple.Item1;
            await _repository.UpdateAsync(db);
            //计算订单均摊成本
            await _orderCostService.ApportionCompute(dto.SpreadlogicId ?? 0,oldDb.LadingBillNumber,dto.LadingBillNumber, orders, tuple.Item2, Model.Enums.SystemBillTypeEnum.LadingBill);
        }
        public async Task RemoveLadingBill(int LadingBillId)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append($@" update a set IsDelete=1 
from Compute_ExpenseAmount(nolock) a
left join Logistisc_Order(nolock) b on b.SystemNo=a.Number
left join Finance_Logistics_LadingBill(nolock) c on c.LadingBillNumber=b.LadingBillNumber
where a.BillTypeCode=c.BillTypeCode and a.SystemBillType=c.SystemBillType and c.Id={LadingBillId} and a.CostType={(int)CostType.预估成本}  ");
            sb.Append($" update Finance_Logistics_LadingBill set IsDelete=1 where Id={LadingBillId} ");
            await _repository.ExecuteSqlAsync(sb.ToString());
        }

        #endregion
        #region  帐单费项
        public async Task<IEnumerable<Finance_Incidental>> GetIncidentals(int LadingBillId)
        {
            return await _repository.GetIncidentals(LadingBillId);
        }
        #endregion
        #region 范围帐单 增改删
        public async Task<long> ScopeBillCreate(int OrderQuerySettingId)
        {
            //获取执行数据
            var orders = await _orderCostService.GetOrdersByScopeBill(OrderQuerySettingId: OrderQuerySettingId);
            //简单创建，不做任何特殊处理
            decimal weight = orders.Sum(x => x.OPWeight);
            Finance_Logistics_LadingBill toDb = new Finance_Logistics_LadingBill()
            {
                LadingBillNumber = "SMPSCOPE_" + DateTime.Now.ToString("yyyyMMddHHmmssffff"),
                BoxQty = orders.Where(x => !string.IsNullOrEmpty(x.BoxNumber))?.Distinct()?.Count() ?? 0,
                Pieces = orders?.Count() ?? 0,
                ChargeWeight = weight,
                Weight = weight,
                OrderQuerySettingId = OrderQuerySettingId,
                BillTime = DateTime.Now,
                BillTypeCode = "",
                SystemBillType = Model.Enums.SystemBillTypeEnum.ScopeBill,
            };
            return await _repository.InsertAsync(toDb);
        }
        public async Task ScopeBillEdit(LadingBillDto dto)
        {
            var oldDb = await _repository.GetAsync(dto.Id);
            var orders = await _orderCostService.GetOrdersByScopeBill(dto.LadingBillNumber);
            await _repository.ClearIncidentals(dto.Id);
            var db = dto.Copy<Finance_Logistics_LadingBill>();
            IncidentalsCompute compute = new IncidentalsCompute() { LadingBillId = dto.Id, LadingBillNumber = dto.LadingBillNumber, IncidentalFees = dto.finances, Weight = dto.ServicesWeight ?? dto.Weight, Pieces = dto.Pieces ?? 0, orders = orders, BillTime = dto.BillTime, Currency = dto.BillCurrency, _Destination = dto._Destination, M3 = (dto.M3 ?? 0.0M) };
            var tuple = await _orderCostService.AddIncidentals(compute);
            db.Cost = tuple.Item1;
            await _repository.UpdateAsync(db);
            //计算订单均摊成本
            await _orderCostService.ApportionCompute(dto.SpreadlogicId ?? 0, oldDb.LadingBillNumber,dto.LadingBillNumber, orders, tuple.Item2, Model.Enums.SystemBillTypeEnum.ScopeBill);
        }
        public async Task RemoveScopeBill(int LadingBillId)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append($@" update a set IsDelete=1 
from Compute_ExpenseAmount(nolock) a
left join Finance_OrderRangeSearchResult(nolock) b1 on b1.SystemNo=a.Number
left join Finance_Logistics_LadingBill(nolock) c on c.OrderQuerySettingId=b1.RangeSettingId
where a.BillTypeCode=c.BillTypeCode and a.SystemBillType=c.SystemBillType and c.Id={LadingBillId} and a.CostType={(int)CostType.预估成本}  ");
            sb.Append($" update Finance_Logistics_LadingBill set IsDelete=1 where Id={LadingBillId} ");
            await _repository.ExecuteSqlAsync(sb.ToString());
        }
        #endregion
        #region 客户头程帐单增删改
        public async Task<long> CustomerLadingBillCreate(LadingBillDto dto)
        {
            if (await _repository.IsExistsAsync(x => x.LadingBillNumber == dto.LadingBillNumber))
            {
                throw new Exception("已存在相同类型帐单" + dto.LadingBillNumber + "不允许创建新的同类型帐单");
            }
            var db = dto.Copy<Finance_Logistics_LadingBill>();
            int Id = (int)await _repository.InsertAsync(db);
            IncidentalsCompute compute = new IncidentalsCompute() { LadingBillId = Id, LadingBillNumber = dto.LadingBillNumber, IncidentalFees = dto.finances, Weight = dto.ServicesWeight ?? dto.Weight, Pieces = dto.Pieces ?? 0, orders = null, BillTime = dto.BillTime, Currency = dto.BillCurrency, _Destination = dto._Destination, M3 = (dto.M3 ?? 0.0M) };
            var tuple = await _orderCostService.AddIncidentals(compute);
            await _repository.UpdateByPrimaryKeyAsync(Id, "Cost", tuple.Item1);
            //成本写入
            await _orderCostService.ApportionComputeOnlyLading(dto.LadingBillNumber, tuple.Item2, Model.Enums.SystemBillTypeEnum.CustomerLadingBill);
            return Id;
        }
        public async Task CustomerLadingBillEdit(LadingBillDto dto)
        {
            if (dto.Prepaid > 0) { throw new Exception("帐单已付款不可修改！"); }
            var oldDb = await _repository.GetAsync(dto.Id);
            if (oldDb.LadingBillNumber != dto.LadingBillNumber)
            {
                if (await _repository.IsExistsAsync(x => x.LadingBillNumber == dto.LadingBillNumber  && x.Id != dto.Id))
                {
                    throw new Exception("提单号重复，请确认填写是否正确！");
                }
            }
            await _repository.ClearIncidentals(dto.Id);
            var db = dto.Copy<Finance_Logistics_LadingBill>();
            IncidentalsCompute compute = new IncidentalsCompute() { LadingBillId = dto.Id, LadingBillNumber = dto.LadingBillNumber, IncidentalFees = dto.finances, Weight = dto.ServicesWeight ?? dto.Weight, Pieces = dto.Pieces ?? 0, orders = null, BillTime = dto.BillTime, Currency = dto.BillCurrency, _Destination = dto._Destination, M3 = (dto.M3 ?? 0.0M) };
            var tuple = await _orderCostService.AddIncidentals(compute);
            db.Cost = tuple.Item1;
            await _repository.UpdateAsync(db);
            //成本写入
            await _orderCostService.ApportionComputeOnlyLading(dto.LadingBillNumber, tuple.Item2, Model.Enums.SystemBillTypeEnum.CustomerLadingBill);
        }
        public async Task RemoveCustomerLadingBill(int LadingBillId)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append($" update Finance_Logistics_LadingBill set IsDelete=1 where Id={LadingBillId} ");
            await _repository.ExecuteSqlAsync(sb.ToString());
        }
            #endregion
        }
    #endregion
}
