﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Core.Standard.Data.Infrastructure;
using Dapper;
using XYY.Common.Standard;
using SMP.Model.Dto.Finance;
using SMP.Model.Finance;
using SMP.Data.Finance;
using SMP.Service.Compute;
using SMP.Model.Dto.Compute;
using SMP.Model.Compute;
using SMP.Model.Enums;
using SMP.Data.Compute;
using SMP.Data.TimeTask;
using SMP.Model.TimeTask;

namespace SMP.Service.Finance
{
    #region Iservices
    public interface IOrderCostService
    {
        #region 数据查询与同步
        Task<PageResult<Compute_ExpenseAmount>> QueryPage(QueryModel qm);
        Task<IEnumerable<Logistisc_Order>> GetOrdersByLadingBillNumber(string LadingBillNumber);
        Task<IEnumerable<Logistisc_Order>> GetOrdersBySystemNos(IEnumerable<string> SystemNos);
        Task<IEnumerable<Logistisc_Order>> GetOrdersByScopeBill(string LadingBillNumber = "", int OrderQuerySettingId = 0);
        /// <summary>
        /// 同步订单信息保存
        /// </summary>
        /// <param name="orders"></param>
        /// <returns></returns>
        Task<bool> SyncOrderInfo(IEnumerable<Logistisc_Order> orders);
        #endregion
        #region 提单成本计算
        /// <summary>
        /// 计算费用
        /// </summary>
        /// <param name="compute"></param>
        /// <returns></returns>
        public Task<Tuple<decimal, List<ComputeAmountResult>>> AddIncidentals(IncidentalsCompute compute);
        /// <summary>
        /// 摊分费用
        /// </summary>
        /// <param name="SpreadlogicId"></param>
        /// <param name="computes"></param>
        /// <returns></returns>
        Task<bool> ApportionCompute(int SpreadlogicId, string OldLadingBillNumber, string LadingBillNumber, IEnumerable<Logistisc_Order> orders, List<ComputeAmountResult> computes, SystemBillTypeEnum billTypeEnum);
        /// <summary>
        /// 费用记录
        /// </summary>
        /// <returns></returns>
        Task<bool> ApportionComputeOnlyLading(string billNumber, List<ComputeAmountResult> computes, SystemBillTypeEnum billTypeEnum);
        #endregion
        #region 订单成本计算
        Task<bool> EestimateOrderEndCost(Finance_TimeTask TaskInfo, IEnumerable<Logistisc_Order> orders);

        List<Compute_ExpenseAmount> ApportionComputeRecord(ApportionComputeRecordInput input);
        Task<bool> InsertApportionComputeRecord(List<Compute_ExpenseAmount> allApportionComputeAmounts);
        #endregion
    }
    #endregion
    #region services
    public class OrderCostService : IOrderCostService
    {
        private readonly IUnitOfWork _unitOfWork;
        private readonly ICompute_ExpenseAmountRepository _repository;
        private readonly IFinance_Logistics_LadingBillRepository _LadingBillRepository;
        private readonly IFinance_TimeTaskRepository _TimeTaskRepository;

        private readonly IComputeAmountService _computeAmountService;
        private readonly ISpreadlogicService _spreadlogicService;
        private readonly ILogistics_ChannelService _ChannelService;
        private readonly IComputeExpenseService _computeExpenseService;
        private readonly IComputeExpenseItemService _computeExpenseItemService;
        public OrderCostService(IUnitOfWork unitOfWork, ICompute_ExpenseAmountRepository repository, IComputeAmountService computeAmountService, ISpreadlogicService spreadlogicService, ILogistics_ChannelService ChannelService, IComputeExpenseService computeExpenseService
            , IComputeExpenseItemService computeExpenseItemService
            , IFinance_Logistics_LadingBillRepository LadingBillRepository
            , IFinance_TimeTaskRepository TimeTaskRepository
            )
        {
            _unitOfWork = unitOfWork;
            _repository = repository;
            _computeAmountService = computeAmountService;
            _spreadlogicService = spreadlogicService;
            _ChannelService = ChannelService;
            _computeExpenseService = computeExpenseService;
            _computeExpenseItemService = computeExpenseItemService;
            _LadingBillRepository = LadingBillRepository;
            _TimeTaskRepository = TimeTaskRepository;
        }
        #region 数据查询与同步
        public async Task<IEnumerable<Logistisc_Order>> GetOrdersByLadingBillNumber(string LadingBillNumber)
        {
            string sql = $"select * from Logistisc_Order where LadingBillNumber='{LadingBillNumber}' ";
            return await _unitOfWork.QueryBySqlAsync<Logistisc_Order>(sql);
        }
        public async Task<IEnumerable<Logistisc_Order>> GetOrdersByScopeBill(string LadingBillNumber = "", int OrderQuerySettingId = 0)
        {
            StringBuilder sb = new StringBuilder();
            string sqlFrom = string.Empty; string sqlLeftJoin = string.Empty; string sqlOn = string.Empty;
            if (!string.IsNullOrEmpty(LadingBillNumber))
            {
                sqlFrom = " Finance_Logistics_LadingBill(nolock) a ";
                sqlLeftJoin = " left join ";
                sqlOn = " on b.RangeSettingId=a.OrderQuerySettingId ";
            }
            sb.Append(@$" select c.*
from {sqlFrom}
{sqlLeftJoin} Finance_OrderRangeSearchResult(nolock) b {sqlOn}
left join Logistisc_Order(nolock) c on c.SystemNo=b.SystemNo ");
            sb.Append($" where 1=1  ");

            if (!string.IsNullOrEmpty(LadingBillNumber)) { sb.Append($" and a.SystemBillType={(int)SystemBillTypeEnum.ScopeBill} and a.LadingBillNumber='{LadingBillNumber}' "); }
            if (OrderQuerySettingId > 0) { sb.Append($" and b.RangeSettingId={OrderQuerySettingId} "); }
            return await _unitOfWork.QueryBySqlAsync<Logistisc_Order>(sb.ToString());
        }
        public async Task<IEnumerable<Logistisc_Order>> GetOrdersBySystemNos(IEnumerable<string> SystemNos)
        {
            try
            {
                var xml = string.Join("\r\n", SystemNos.Select(x => $"<NumberList><Number>{x}</Number></NumberList>"));
                string systemNoSql = @"
if exists (select * from tempdb.dbo.sysobjects where id = object_id(N'tempdb..#temp_getordersbysystemnos') and type='U')
    drop table #temp_getordersbysystemnos
                                            DECLARE @idoc int
                                            DECLARE @doc varchar(max)
                                            SET @doc ='
                                            <ROOT>
                                            {0}
                                            </ROOT>'
                                            --Create an internal representation of the XML document.
                                            EXEC sp_xml_preparedocument @idoc OUTPUT, @doc
                                            -- SELECT stmt using OPENXML rowset provider
                                            SELECT * into #temp_getordersbysystemnos
                                            FROM   OPENXML (@idoc, '/ROOT/NumberList',2)
                                                     WITH (Number nvarchar(500)) ";
                string selectSql = " select * from Logistisc_Order(nolock) where SystemNo in (select * from #temp_getordersbysystemnos) ";
                string executeSql = systemNoSql.Formater(xml) + selectSql;
                return await _unitOfWork.QueryBySqlAsync<Logistisc_Order>(executeSql);
            }
            catch (Exception ex)
            {
                return null;
            }
        }
        //public async Task<IEnumerable<string>> GetFinanceCostTableName(string BillTypeCode)
        //{
        //    string sql = "Select name  From sysobjects Where name not in ('Finance_Cost_Base','Finance_Cost_End')";
        //    if (!string.IsNullOrEmpty(BillTypeCode)) { sql += $"and name='Finance_Cost_{BillTypeCode}'"; } else { sql += " and name like 'Finance_Cost_%' "; }
        //    return await _unitOfWork.QueryBySqlAsync<string>(sql);
        //}

        public async Task<PageResult<Compute_ExpenseAmount>> QueryPage(QueryModel qm)
        {
            if (qm.QueryParamer == null) { qm.QueryParamer = new List<QueryParamer>(); }
            qm.QueryParamer.Add(new QueryParamer() { Filed = "IsDelete", Value = "0", Method = "Eq" });

            return await _repository.QueryPagerAsync(qm);
            //#region 基础sql
            //StringBuilder sb = new StringBuilder();
            //var tableNams = await GetFinanceCostTableName("");
            //string filed = string.Empty; string leftJoin = string.Empty; int index = 0;
            //if (tableNams.Count() > 0)
            //{
            //    filed = ",(";
            //    foreach (var item in tableNams)
            //    {
            //        filed += $"b{index}.OrderCost"; leftJoin += $" left join {item} b{index} on b{index}.SystemNo=a.SystemNo ";
            //        index++;
            //    }
            //    filed += ") TotalCost";
            //}
            //sb.Append($"select a.*{filed} from Finance_Cost_End a ");
            //sb.Append($" {leftJoin} ");
            //#endregion
            //#region 分页
            ////排序
            //string sortString = string.Empty;
            //foreach (var item in qm.QuerySort)
            //{
            //    sortString += " " + item.Field + " " + (item.IsDesc ? "DESC" : "") + ",";
            //}
            //sortString = sortString.TrimEnd(',');
            //string sql1 = sb.ToString();
            //StringBuilder pager = new StringBuilder();
            //string where = string.Empty;
            //if (qm.QueryParamer != null) { where = _unitOfWork.GetWhere(qm.QueryParamer); }
            //pager.Append($@"select * from (select Row_Number()over(order by  {sortString})rowId,*
            //from ({sql1})a where 1=1 {where}) T where rowId between {(qm.PageIndex - 1) * qm.PageSize + 1} and {qm.PageSize * qm.PageIndex};");
            //pager.Append(@$"select count(0)TotalCount from (select Row_Number()over(order by  {sortString})rowId,*
            //from({ sql1})a where 1 = 1 { where}) T where 1=1");
            //#endregion
            //#region 取值
            //string sql = pager.ToString();
            //var dr = await _unitOfWork.Connection.QueryMultipleAsync(sql);
            ////需要注意读取顺序，依据语句执行顺序进行读取
            //var pageResult = new PageResult<OrderCost>()
            //{
            //    Result = dr.Read<OrderCost>(),
            //    PageIndex = qm.PageIndex,
            //    PageSize = qm.PageIndex,
            //    TotalCount = dr.Read<int>().FirstOrDefault()
            //};
            //#endregion
            //return pageResult;
        }

        public async Task<bool> SyncOrderInfo(IEnumerable<Logistisc_Order> orders)
        {
            var batchQty = 5000.0;
            try
            {
                for (int i = 0; i < Math.Ceiling(orders.Count() / batchQty); i++)
                {
                    var tempOrders = orders.Skip(i * (int)batchQty).Take((int)batchQty).ToList();
                    var xml = string.Join("\r\n", tempOrders.Select(x => $"<NumberList><Number>{x.SystemNo}</Number></NumberList>"));
                    string systemNoSql = @"
if exists (select * from tempdb.dbo.sysobjects where id = object_id(N'tempdb..#temp_searchs') and type='U')
    drop table #temp_searchs
                                            DECLARE @idoc int
                                            DECLARE @doc varchar(max)
                                            SET @doc ='
                                            <ROOT>
                                            {0}
                                            </ROOT>'
                                            --Create an internal representation of the XML document.
                                            EXEC sp_xml_preparedocument @idoc OUTPUT, @doc
                                            -- SELECT stmt using OPENXML rowset provider
                                            SELECT * into #temp_searchs
                                            FROM   OPENXML (@idoc, '/ROOT/NumberList',2)
                                                     WITH (Number nvarchar(500)) ";
                    string deleteSql = " delete Logistisc_Order where SystemNo in (select * from #temp_searchs) ";
                    string executeSql = systemNoSql.Formater(xml) + deleteSql;
                    await _unitOfWork.ExecuteAsync(executeSql);
                    await _unitOfWork.BulkToDBAsync<Logistisc_Order>(tempOrders.ToList());
                }
            }
            catch (Exception ex)
            {

                return false;
            }
            return true;

        }
        #endregion
        #region 提单成本计算
        /// <summary>
        /// 提单成本合计
        /// </summary>
        /// <returns></returns>
        public async Task<Tuple<decimal, List<ComputeAmountResult>>> AddIncidentals(IncidentalsCompute compute)
        {
            decimal sumIncidentalFee = 0;
            List<ComputeAmountResult> computes = new List<ComputeAmountResult>();
            List<Finance_Incidental> toDb = new List<Finance_Incidental>();
            if (compute.IncidentalFees != null)
            {
               var temp= compute.IncidentalFees.ToList();
                temp.RemoveAll(x => x.IncidentalType == 0);
                List<ComputeAmountSouce> souces = new List<ComputeAmountSouce>();
                compute.IncidentalFees = await FinancesVerify(compute.LadingBillId, compute.IncidentalFees);
                foreach (var fa in temp)
                {
                    var tempList = fa.ExpenseResult.Where(x => x.num == 1).Select(x => { x.IncidentalFee = 0; return x; }).CopyList<Finance_Incidental>();//重置数值,去重
                    //var disTempList = tempList.GroupBy(x=>new {x.ExpenseId,x.VersionId }).Select(x=>x.FirstOrDefault()).ToList();//去掉展示需要的数据，仅保留费项关系
                    foreach (var item in tempList)
                    {
                        item.ExpenseItemId = fa.IncidentalType;
                        item.LadingBillId = compute.LadingBillId;
                    }
                    toDb.AddRange(tempList);
                    souces.Add(new ComputeAmountSouce()
                    {
                        ComputeDate = compute.BillTime,
                        BindBillId = compute.LadingBillId,
                        Weight = compute.Weight,
                        ExpenseItemId = fa.IncidentalType,
                        Currency = compute.Currency,
                        BillNumber = "",
                        CountryCode = "",
                        State = "",
                        City = "",
                        ZipCode = "",
                        Stree = "",
                        TrackingNumber = "",
                        Port = compute._Destination,
                        OrderQty = compute.Pieces,
                        M3 = compute.M3,
                    });
                }
                computes = await _computeAmountService.MultipleCompute(souces);
                //总值计算
                if (computes.Any(x => !x.IsSuccess))
                {
                    var errorInfo = computes.Where(x => !x.IsSuccess).Select(x => x.ExpenseName + ":" + x.Message).Distinct();
                    throw new Exception(string.Join(',', errorInfo));
                }
                sumIncidentalFee = computes.Sum(x => x.Amount);
                if (toDb.Count > 0) { await _unitOfWork.BulkToDBAsync<Finance_Incidental>(toDb); }
            }
            return Tuple.Create(sumIncidentalFee, computes);
        }
        public async Task<bool> ApportionCompute(int SpreadlogicId, string OldLadingBillNumber, string LadingBillNumber, IEnumerable<Logistisc_Order> orders, List<ComputeAmountResult> computes, SystemBillTypeEnum billTypeEnum)
        {
            //基础数据判断 start
            if (!await ClearApportionComputeRecord(new List<string>() { OldLadingBillNumber }, (int)CostType.预估成本)) { throw new Exception("旧数据清除失败！"); }
            var logicDetails = await _spreadlogicService.GetSpreadlogicDetails(SpreadlogicId);
            if (logicDetails == null || logicDetails.Count() == 0 || computes == null || computes.Count == 0) { return false; }
            var DetailIds = computes.Select(x => x.DetailId);
            logicDetails = logicDetails.Where(x => x.ApportionMethod != Model.Enums.ApportionMethodEnum.None && DetailIds.Contains(x.ExpenseId));
            if (logicDetails == null || logicDetails.Count() == 0) { return false; }

            var channelId = orders.Select(x => x.OrderChannelUniqueId).Distinct().ToList();
            var channels = await _ChannelService.GetAllChannelList(channelId);
            if (channels == null || channels.Count() == 0) { throw new Exception("未获取到订单相应的渠道！"); }
            //基础数据判断 end
            var TotalWegith = orders.Sum(x => x.OPWeight);
            orders.ToList().ForEach(x => x.WeightRate = decimal.Round(Convert.ToDecimal(x.OPWeight) / Convert.ToDecimal(TotalWegith), 6));//重量占比保留六位
            List<Compute_ExpenseAmount> allApportionComputeAmounts = new List<Compute_ExpenseAmount>();
            foreach (var item in logicDetails)
            {
                var amountResult = computes.Where(x => x.DetailId == item.ExpenseId).FirstOrDefault();
                ApportionComputeRecordInput input = new ApportionComputeRecordInput()
                {
                    orders = orders,
                    detail = item,
                    LadingBillNumber= LadingBillNumber,
                    amountResult = amountResult,
                    SystemBillType = billTypeEnum,
                };
                var computeAmounts = ApportionComputeRecord(input);
                allApportionComputeAmounts.AddRange(computeAmounts);
            }
            //创建异步任务
            if (billTypeEnum != SystemBillTypeEnum.ScopeBill)
            {
                orders = await PortVerify(orders, channels);
                await _TimeTaskRepository.CreateTypeTaskByPort(orders.Where(x => !string.IsNullOrEmpty(x.Port)));
            }

            return await InsertApportionComputeRecord(allApportionComputeAmounts);
        }
        public async Task<bool> ApportionComputeOnlyLading(string billNumber, List<ComputeAmountResult> computes, SystemBillTypeEnum billTypeEnum)
        {
            bool result = true;
            List<string> billNumbers = new List<string>() { billNumber};
            //基础数据判断 start
            if (!await ClearApportionComputeRecord(billNumbers, (int)CostType.预估成本)) { throw new Exception("旧数据清除失败！"); }
            List<Compute_ExpenseAmount> allApportionComputeAmounts = new List<Compute_ExpenseAmount>();
            foreach (var compute in computes)
            {
                DateTime TransactionTime = DateTime.Now;
                allApportionComputeAmounts.Add(new Compute_ExpenseAmount()
                {
                    Number = billNumber,
                    ExpenseId = compute.ExpenseId,
                    ExpenseDetailId = compute.DetailId,
                    VersionId = compute.VersionId,
                    ExpenseItemId = compute.ExpenseItemId,
                    TransactionTime = TransactionTime,
                    Currency = compute.BillCurrency,
                    AmountinOriginalCurrency = compute.AmountinOriginalCurrency,
                    SettlementRate = compute.SettlementRate,
                    SettlementCurrency = compute.Currency,
                    SettlementAmount = compute.Amount,
                    FeeDescription = compute.ExpenseName,
                    LadingBillNumber=billNumber,
                    CostType = CostType.预估成本,
                    CreateTime = DateTime.Now,
                    Zone = compute.Zones,
                    IsDelete = false,
                    SystemBillType = billTypeEnum,
                });
            }
            await InsertApportionComputeRecord(allApportionComputeAmounts);
            return result;
        }
        #endregion
        #region 订单成本计算
        public async Task<bool> EestimateOrderEndCost(Finance_TimeTask TaskInfo, IEnumerable<Logistisc_Order> orders)
        {
            bool result = true;
            //计算的基础数据获取 start
            var channelId = orders.Select(x => x.OrderChannelUniqueId).Distinct().ToList();
            var channels = await _ChannelService.GetAllChannelList(channelId);
            var batchQty = 1000.0;
            //计算的基础数据获取 end
            if (channels == null || channels.Count() == 0) { throw new Exception("未获取到订单相应的渠道！"); }
            orders = await PortVerify(orders, channels);
            List<Compute_ExpenseAmount> allApportionComputeAmounts = new List<Compute_ExpenseAmount>();
            for (int i = 0; i < Math.Ceiling(orders.Count() / batchQty); i++)
            {
                var tempOrders = orders.Skip(i * (int)batchQty).Take((int)batchQty).ToList();
                List<ComputeAmountResult> computes = new List<ComputeAmountResult>();
                List<ComputeAmountSouce> souces = new List<ComputeAmountSouce>();
                if (!await ClearApportionComputeRecord(orders,SystemBillTypeEnum.OrderBill, (int)CostType.预估成本)) { throw new Exception("旧数据清除失败！"); }//独立订单计算 固定类型
                foreach (var item in tempOrders)
                {
                    var tempChannel = channels.Where(x => x.ThirdPartyUniqueId == item.OrderChannelUniqueId).FirstOrDefault();
                    if (tempChannel == null) { continue; }
                    //总费用计算
                    souces.Add(new ComputeAmountSouce()
                    {
                        ComputeDate = item.OrderTime,
                        Port = item.Port,
                        Weight = item.OPWeight,
                        Currency = tempChannel.BillCurrency,
                        Volume = (item.Length * item.Width * item.Height) ?? 0,
                        ExpenseItemId = tempChannel.ExpenseItemId ?? 0,
                        BillNumber = item.SystemNo,
                        CountryCode = item.ReceiverCountryCode,
                        State = item.ReceiverState,
                        City = item.ReceiverCity,
                        ZipCode = item.ReceiverZipCode,
                        Stree = item.ReceiverStreet,
                        TrackingNumber = item.TrackingNumber,
                    });
                    item.WeightRate = 1;//单个订单预估重量占比始终为1
                }
                computes = await _computeAmountService.MultipleCompute(souces);
                //计算失败数据处理
                if (computes.Any(x => !x.IsSuccess))
                {
                    result = false;
                    await _TimeTaskRepository.CreateTypeTaskScopeDetail(TaskInfo, computes.Where(x => !x.IsSuccess).ToList());
                    computes.RemoveAll(x => !x.IsSuccess);
                }
                //处理后数据写入
                foreach (var compute in computes)
                {
                    DateTime TransactionTime = DateTime.Now;
                    allApportionComputeAmounts.Add(new Compute_ExpenseAmount()
                    {
                        Number = compute.BillNumber,
                        ExpenseId = compute.ExpenseId,
                        ExpenseDetailId = compute.DetailId,
                        VersionId = compute.VersionId,
                        ExpenseItemId = compute.ExpenseItemId,
                        TransactionTime = TransactionTime,
                        Currency = compute.BillCurrency,
                        AmountinOriginalCurrency = compute.AmountinOriginalCurrency,
                        SettlementRate = compute.SettlementRate,
                        SettlementCurrency = compute.Currency,
                        SettlementAmount = compute.Amount,
                        FeeDescription = compute.ExpenseName,
                        BillTypeCode = "SystemEndBill",
                        CostType = CostType.预估成本,
                        CreateTime = DateTime.Now,
                        Zone = compute.Zones,
                        IsDelete = false,
                        SystemBillType = SystemBillTypeEnum.OrderBill,
                    });
                }
            }
            await InsertApportionComputeRecord(allApportionComputeAmounts);
            return result;
        }
        #endregion
        #region 数据落地
        public List<Compute_ExpenseAmount> ApportionComputeRecord(ApportionComputeRecordInput input)
        {
            bool result = true; bool isWeightRate = false;
            decimal ApportionValue = 0.0M;//结算币种分摊
            decimal AmountinOriginalCurrency = 0.0M;//原币种分
            switch (input.detail.ApportionMethod)
            {
                case Model.Enums.ApportionMethodEnum.OrderCount:
                    ApportionValue = input.amountResult.Amount / input.orders.Count();
                    AmountinOriginalCurrency = input.amountResult.AmountinOriginalCurrency / input.orders.Count();
                    break;
                case Model.Enums.ApportionMethodEnum.Actually:
                    ApportionValue = input.amountResult.Amount;
                    AmountinOriginalCurrency = input.amountResult.AmountinOriginalCurrency;
                    break;
                case Model.Enums.ApportionMethodEnum.WeightRate:
                    ApportionValue = input.amountResult.Amount;
                    AmountinOriginalCurrency = input.amountResult.AmountinOriginalCurrency;
                    isWeightRate = true;
                    break;
            }
            List<Compute_ExpenseAmount> toDbModel = new List<Compute_ExpenseAmount>();
            DateTime TransactionTime = DateTime.Now;// Convert.ToDateTime(DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss:ffff"));
            foreach (var item in input.orders)
            {
                toDbModel.Add(new Compute_ExpenseAmount()
                {
                    Number = item.SystemNo,
                    ExpenseId = input.detail.ExpenseId,
                    ExpenseDetailId = input.amountResult.DetailId,
                    VersionId = input.amountResult.VersionId,
                    ExpenseItemId = input.detail.ExpenseItemId,
                    TransactionTime = TransactionTime,
                    Currency = input.amountResult.BillCurrency,
                    AmountinOriginalCurrency = !isWeightRate ? AmountinOriginalCurrency : AmountinOriginalCurrency * (decimal)item.WeightRate,
                    SettlementRate = input.amountResult.SettlementRate,
                    SettlementCurrency = input.amountResult.Currency,
                    SettlementAmount = !isWeightRate ? ApportionValue : ApportionValue * (decimal)item.WeightRate,
                    FeeDescription = input.amountResult.ExpenseName,
                    LadingBillNumber = input.LadingBillNumber,
                    CostType = input.CostType,
                    CreateTime = DateTime.Now,
                    SystemBillType = input.SystemBillType,
                });
            }

            return toDbModel;
        }
        public async Task<bool> InsertApportionComputeRecord(List<Compute_ExpenseAmount> allApportionComputeAmounts)
        {
            bool insertOk = false;
            if (allApportionComputeAmounts?.Any() == true)
            {
                decimal batchQty = 5000.0M;//5000条一批
                for (int i = 0; i < Math.Ceiling(allApportionComputeAmounts.Count() / batchQty); i++)
                {
                    var temp = allApportionComputeAmounts.Skip(i * (int)batchQty).Take((int)batchQty).ToList();
                    insertOk = await _unitOfWork.BulkToDBAsync<Compute_ExpenseAmount>(temp);
                }
            }
            return insertOk;
        }
        /// <summary>
        /// 末端成本数据记录清理
        /// </summary>
        /// <returns></returns>
        public async Task<bool> ClearApportionComputeRecord(IEnumerable<Logistisc_Order> orders, SystemBillTypeEnum billTypeEnum, int costType)
        {
            if (orders==null||orders.Count()==0) { return true; }
            var batchQty = 5000.0;
            try
            {
                for (int i = 0; i < Math.Ceiling(orders.Count() / batchQty); i++)
                {
                    var tempOrders = orders.Skip(i * (int)batchQty).Take((int)batchQty).ToList();
                    var xml = string.Join("\r\n", tempOrders.Select(x => $"<NumberList><Number>{x.SystemNo}</Number></NumberList>"));
                    string systemNoSql = @"
if exists (select * from tempdb.dbo.sysobjects where id = object_id(N'tempdb..#temp_searchs') and type='U')
    drop table #temp_searchs
                                            DECLARE @idoc int
                                            DECLARE @doc varchar(max)
                                            SET @doc ='
                                            <ROOT>
                                            {0}
                                            </ROOT>'
                                            --Create an internal representation of the XML document.
                                            EXEC sp_xml_preparedocument @idoc OUTPUT, @doc
                                            -- SELECT stmt using OPENXML rowset provider
                                            SELECT * into #temp_searchs
                                            FROM   OPENXML (@idoc, '/ROOT/NumberList',2)
                                                     WITH (Number nvarchar(500)) ";
                    string deleteSql = $" delete Compute_ExpenseAmount where Number in (select * from #temp_searchs)  and SystemBillType={(int)billTypeEnum} and costType={costType}";
                    string updateToDeleteSql = $"UPDATE Compute_ExpenseAmount set IsDelete=1 where Number in (select * from #temp_searchs)  and SystemBillType={(int)billTypeEnum} and costType={costType}";

                    string executeSql = systemNoSql.Formater(xml) + updateToDeleteSql;//软删除
                    await _unitOfWork.ExecuteAsync(executeSql);
                }
            }
            catch (Exception ex)
            {
                return false;
            }
            return true;
        }
        /// <summary>
        /// 提单成本数据清理
        /// </summary>
        /// <returns></returns>
        public async Task<bool> ClearApportionComputeRecord(IEnumerable<string> billNumber, int costType)
        {
            if (billNumber == null || billNumber.Count() == 0) { return true; }
            try
            {
                string updateToDeleteSql = $"UPDATE Compute_ExpenseAmount set IsDelete=1 where LadingBillNumber in @billNumber  and costType={costType}";
                string executeSql = updateToDeleteSql;//软删除
                await _unitOfWork.ExecuteAsync(executeSql, new { billNumber = billNumber });
            }
            catch (Exception ex)
            {
                return false;
            }
            return true;
        }
        #endregion
        #region 校验或扩展
        /// <summary>
        /// 费用数据验证
        /// </summary>
        /// <param name="BillId"></param>
        /// <param name="finances"></param>
        /// <returns></returns>
        public async Task<IEnumerable<finances>> FinancesVerify(int BillId, IEnumerable<finances> finances)
        {
            //获取所有待修改数据
            List<Compute_Expense> computes = new List<Compute_Expense>();
            foreach (var item in finances)
            {
                var tempResult = item.ExpenseResult.Where(x => x.num == 1).Select(x => new { x.ExpenseId, x.VersionId }).ToList();
                foreach (var result in tempResult)
                {
                    var Compute_Expense = await _computeExpenseService.GetVersion(result.VersionId);
                    var detailZoon = Compute_Expense.Details.Where(x => x.ExpenseID == result.ExpenseId).FirstOrDefault();
                    if (detailZoon.ZoneValue.Count != 1) { throw new Exception(" 当前费用无分区值，或分区数量大于1！"); }
                    bool isChange = false;
                    foreach (var detail in item.ExpenseResult.Where(x => x.ExpenseId == result.ExpenseId))
                    {
                        var tempDetail = Compute_Expense.Details.Where(x => x.Id == detail.ExpenseDetailId)?.FirstOrDefault();
                        if (tempDetail.ZoneValue.FirstOrDefault().Fee != detail.IncidentalFee)
                        {
                            isChange = true; break;
                        }
                    }
                    if (isChange)
                    {
                        Compute_Expense.BindBillId = BillId;
                        foreach (var detail in Compute_Expense.Details)
                        {
                            detail.ZoneValue.FirstOrDefault().Fee = item.ExpenseResult.Where(x => x.ExpenseDetailId == detail.Id).FirstOrDefault().IncidentalFee;
                        }
                        computes.Add(Compute_Expense);
                    }
                }
            }
            //有修改项进行处理
            if (computes.Count > 0)
            {
                Dictionary<int, int> expen = new Dictionary<int, int>();
                foreach (var item in computes)
                {
                    int versionId = await _computeExpenseService.AddComputeVersionExpense(item);
                    expen.Add(item.Id, versionId);
                }
                //将新的数据记录返回 VersionId
                List<int> ExpenseId = computes.Select(x => x.Id).Distinct().ToList();
                foreach (var item in finances)
                {
                    var tempExpenseResult = item.ExpenseResult.Where(x => ExpenseId.Contains(x.ExpenseId));
                    foreach (var result in tempExpenseResult)
                    {
                        result.VersionId = expen.Where(y => y.Key == result.ExpenseId).FirstOrDefault().Value;
                    }
                }
            }
            return finances;
        }
        /// <summary>
        /// 订单口岸验证
        /// </summary>
        /// <param name="originalData"></param>
        /// <returns></returns>
        public async Task<IEnumerable<Logistisc_Order>> PortVerify(IEnumerable<Logistisc_Order> originalData, IEnumerable<Logistics_Channel> channels)
        {
            List<int> expenseItemIds = channels.Select(x => x.ExpenseItemId ?? 0).Distinct().ToList();
            var portExpenseItems = await _computeExpenseItemService.GetExpenseItemByZoneMethodMode(ZoneMode.提单落地口岸, expenseItemIds);
            if (portExpenseItems == null || portExpenseItems.Count() == 0) { return originalData; }
            List<int> portExpenseItemIds = portExpenseItems.Select(x => x.Id).Distinct().ToList();
            var tempChannel = channels.Where(x => portExpenseItemIds.Contains(x.ExpenseItemId ?? 0)).ToList();
            foreach (var item in tempChannel)
            {
                var tempOrders = originalData.Where(x => x.OrderChannelUniqueId == item.ThirdPartyUniqueId);
                //考虑放到循环外取数
                List<string> LadingBillNumbers = tempOrders.Select(x => x.LadingBillNumber ?? "").Distinct().ToList();
                var ladingBills = await _LadingBillRepository.GetPortLadingBillsByNumbers(LadingBillNumbers);

                foreach (var order in tempOrders)
                {
                    order.Port = ladingBills.Where(x => x.LadingBillNumber == order.LadingBillNumber)?.FirstOrDefault()?._Destination ?? "";
                }
            }
            return originalData;
        }
        #endregion
    }
    #endregion

}
