﻿using SMP.Data;
using SMP.Data.Compute;
using SMP.Data.Finance;
using SMP.Data.Logistisc;
using SMP.Model;
using SMP.Model.Compute;
using SMP.Model.Dto.Compute;
using SMP.Model.Dto.Finance;
using SMP.Model.Enums;
using SMP.Model.Finance;
using SMP.Service.Finance.Dto;
using SMP.Service.OrderRangeQuery;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SMP.Service.Finance
{
    public interface IServiceBillSpreadService
    {
        Task SpreadServiceBill(SpreadServiceBillParam param);
        Task<bool> IsServiceBillOrderRangeSpread(string billName);
    }
    public class ServiceBillSpreadService : IServiceBillSpreadService
    {
        IOrderRangeQueryService _orderRangeQueryService;
        IFinance_ServiceBillResolveSolutionRepository _ServiceBillResolveSolutionRepository;
        IFinance_OrderQuerySettingRepository _orderQuerySettingRepository;
        IFinance_OrderRangeSearchResultRepository _OrderRangeSearchResultRepository;
        IFinance_ServiceBillResolveResultRepository _ServiceBillResolveResultRepository;
        IFinance_ServiceBillShareAmountRepository _ServiceBillShareAmountRepository;
        IFinance_SpreadlogicRepository _SpreadlogicRepository;
        IFinance_SpreadlogicDetailRepository _SpreadlogicDetailRepository;
        IFinance_ServiceBillExpenseMappingSolutionRepository _expenseMappingSolutionRepository;
        IFinance_ServiceBillExpenseMappingRepository _expenseMappingRepository;
        IOrderCostService _orderCostService;
        ISysRateService _rateService;
        IView_ServiceBillResultOrderInfoRepository _serviceBillResultOrderInfoRepository;
        IView_ChannelSpreadLogicDetailRepository _channelSpreadLogicDetailRepository;
        ICompute_ExpenseRepository _expenseRepository;
        IFinance_Logistics_LadingBillRepository _ladingBillRepository;
        IView_OrderRangeOrderInfoRepository _orderRangeOrderInfoRepository;
        IFinance_ServiceBillResolveApplyRepository _serviceBillResolveApplyRepository;
        IServiceBillSpreadApplyService _serviceBillSpreadApplyService;

        public ServiceBillSpreadService(IOrderRangeQueryService orderRangeQueryService, IFinance_OrderQuerySettingRepository orderQuerySettingRepository,
            IFinance_OrderRangeSearchResultRepository orderRangeSearchResultRepository, IFinance_ServiceBillResolveResultRepository ServiceBillResolveResultRepository
            , IFinance_ServiceBillShareAmountRepository serviceBillShareAmountRepository,
            IFinance_SpreadlogicRepository spreadlogicRepository, IFinance_SpreadlogicDetailRepository spreadlogicDetailRepository,
            IFinance_ServiceBillExpenseMappingSolutionRepository expenseMappingSolutionRepository, IFinance_ServiceBillExpenseMappingRepository expenseMappingRepository,
            IOrderCostService orderCostService, ISysRateService rateService, IView_ServiceBillResultOrderInfoRepository serviceBillResultOrderInfoRepository,
            IView_ChannelSpreadLogicDetailRepository channelSpreadLogicRepository, ICompute_ExpenseRepository expenseRepository, IFinance_Logistics_LadingBillRepository ladingBillRepository,
            IView_OrderRangeOrderInfoRepository orderRangeOrderInfoRepository, IFinance_ServiceBillResolveApplyRepository serviceBillResolveApplyRepository, IServiceBillSpreadApplyService serviceBillSpreadApplyService
            )
        {
            _orderRangeQueryService = orderRangeQueryService;
            _orderQuerySettingRepository = orderQuerySettingRepository;
            _OrderRangeSearchResultRepository = orderRangeSearchResultRepository;
            _ServiceBillResolveResultRepository = ServiceBillResolveResultRepository;
            _ServiceBillShareAmountRepository = serviceBillShareAmountRepository;
            _SpreadlogicRepository = spreadlogicRepository;
            _SpreadlogicDetailRepository = spreadlogicDetailRepository;
            _expenseMappingSolutionRepository = expenseMappingSolutionRepository;
            _expenseMappingRepository = expenseMappingRepository;
            _orderCostService = orderCostService;
            _rateService = rateService;
            _serviceBillResultOrderInfoRepository = serviceBillResultOrderInfoRepository;
            _channelSpreadLogicDetailRepository = channelSpreadLogicRepository;
            _expenseRepository = expenseRepository;
            _ladingBillRepository = ladingBillRepository;
            _orderRangeOrderInfoRepository = orderRangeOrderInfoRepository;
            _serviceBillResolveApplyRepository = serviceBillResolveApplyRepository;
            _serviceBillSpreadApplyService = serviceBillSpreadApplyService;
        }

        private async Task<IEnumerable<Finance_ServiceBillExpenseMapping>> GetMappings(int serviceProviderId,string billTypeCode)
        {
            var mappingSolutions = await _expenseMappingSolutionRepository.QueryAsync(m => m.ServiceProviderId == serviceProviderId && m.BillTypeCode == billTypeCode &&m.Enabled==true);
            if (mappingSolutions?.Any() != true)
            {
                throw new Exception($"找不到该账单类型(供应商代码:{serviceProviderId},账单类型代码:{billTypeCode})对应的费用映射方案");
            }
            if(mappingSolutions.Count()>1)
            {
                throw new Exception($"账单类型(供应商代码:{serviceProviderId},账单类型代码:{billTypeCode})存在多个可用的费用映射方案");
            }
            var mappingSolution = mappingSolutions.First();

            var mappings = await _expenseMappingRepository.QueryAsync(m => m.MappingSolutionId == mappingSolution.Id);
            if (mappings?.Any() != true)
                throw new Exception($"找不到费用映射明细,ServiceProviderId:{ serviceProviderId},BillTypeCode:{billTypeCode}");

            return mappings;
        }
        public async Task SpreadServiceBill(SpreadServiceBillParam param)
        {
            //账单摊分思路:
            //1.如果账单解析结果的单号类型为跟踪号/系统单号/转单号，则视为末端派送账单，
            //末端派送账单的摊分逻辑已经和渠道关联，所以这部分账单数据通过订单渠道获取摊分逻辑进行摊分。
            //2.如果账单解析结果的单号类型为提单，则视为非末端账单(提单账单)，这类账单的摊分逻辑是关联在提单上的，
            //因此直接在提单上获取摊分方案进行摊分
            //3.如果是无单号类型的账单，则需要指定订单范围搜索设置,并通过指定摊分逻辑进行摊分。

            ValidateParam(param);
            var spreadApplyId = await _serviceBillSpreadApplyService.CheckAndAddNewApply(param.BillName, param.OrderRangeSettingId, param.SpreadLogicId);
            try
            {
                var billResolveResults = await _serviceBillResultOrderInfoRepository.QueryAsync(i => i.BillName == param.BillName && i.DataType == ServiceBillDataType.金额);
                if ((param.OrderRangeSettingId == null || param.OrderRangeSettingId == 0) && billResolveResults?.Any() != true)
                {
                    throw new Exception($"找不到账单解析结果");
                }


                //1.以订单号的每种种费用类型做摊分,无需均摊逻辑，直接转换为费用金额
                var orderNumberResults = billResolveResults.Where(i => i.NumberType == NumberType.系统单号 || i.NumberType == NumberType.跟踪号 || i.NumberType == NumberType.转单号);
                if (orderNumberResults?.Any() == true)
                {
                    await SpreadServiceBillByOrderNumber(orderNumberResults, (int)spreadApplyId);
                }

                //2.以提单号的每种费用类型做摊分
                var ladingBillNumberResults = billResolveResults.Where(i => i.NumberType == NumberType.提单号);
                if (ladingBillNumberResults?.Any() == true)
                {
                    await SpreadServiceBillByLadingBill(ladingBillNumberResults, (int)spreadApplyId);
                }

                //3.以订单范围均摊
                var noneNumberResults = await _ServiceBillResolveResultRepository.QueryAsync(i => i.BillName == param.BillName && i.NumberType == NumberType.无);
                if (noneNumberResults?.Any() == true)
                {
                    if (param.OrderRangeSettingId == null || param.OrderRangeSettingId.Value == 0)
                    {
                        throw new Exception("该账单包含无单号的汇总费用，但未指定订单搜索范围方案");
                    }
                    if (param.SpreadLogicId == null || param.SpreadLogicId.Value == 0)
                    {
                        throw new Exception("该账单包含无单号的汇总费用，但未指定摊分逻辑");
                    }
                    await SpreadServiceBillByOrderRange(noneNumberResults, param.OrderRangeSettingId.Value, param.SpreadLogicId.Value, (int)spreadApplyId);
                }

                var boxNumberResult = billResolveResults.Where(i => i.NumberType == NumberType.箱号);
                if (boxNumberResult?.Any() == true)
                {
                    throw new Exception("暂不支持账单号码为箱号的均摊");
                }

                await _serviceBillSpreadApplyService.UpdateApplySuccess((int)spreadApplyId);
            }
            catch (Exception ex)
            {
                await _serviceBillSpreadApplyService.UpdateApplyError((int)spreadApplyId, ex.Message);
                throw;
            }
        }

        private async Task SpreadServiceBillByOrderRange(IEnumerable<Finance_ServiceBillResolveResult> noneNumberResults, int orderRangeSettingId, int spreadLogicId, int spreadApplyId)
        {
            var currencys = noneNumberResults.Select(i => i.Currency).Distinct();
            var sysRates = await _rateService.GetRates(currencys);

            var allSpreadLogicDetails = await _SpreadlogicDetailRepository.QueryAsync(i => i.SpreadlogicId == spreadLogicId);
            if (allSpreadLogicDetails?.Any() != true)
            {
                throw new Exception($"找不到摊分逻辑明细,摊分逻辑Id:{spreadLogicId}");
            }

            Finance_OrderQuerySetting orderRangeSetting = await _orderQuerySettingRepository.GetAsync(orderRangeSettingId);
            if (orderRangeSetting == null)
            {
                throw new Exception($"找不到设置信息,Id:{orderRangeSettingId}");
            }

            if (orderRangeSetting.IsExecuted == null || orderRangeSetting.IsExecuted.Value == false)
            {
                int orderCount = await _orderRangeQueryService.InsertOrderRange(orderRangeSetting);
                if (orderCount <= 0)
                {
                    throw new Exception($"订单范围搜索结果为0,Id:{orderRangeSettingId}");
                }
            }
            var rangeOrderInfos = await _orderRangeOrderInfoRepository.QueryAsync(i => i.RangeSettingId == orderRangeSettingId);
            if (rangeOrderInfos?.Any() != true)
                throw new Exception($"订单范围搜索关联的订单数为零");

            var dataNameGroups = noneNumberResults.GroupBy(i => i.BillDataName);

            List<View_ServiceBillResultOrderInfo> allBillResoveOrderInfos = new List<View_ServiceBillResultOrderInfo>();
            foreach (var dataNameGroup in dataNameGroups)
            {
                int dataNameGroupCount = dataNameGroup.Count();
                if (dataNameGroupCount > 1)
                {
                    throw new Exception($"账单解析结果中存在{dataNameGroupCount}条名为【{dataNameGroup.Key}】的费用");
                }
                foreach (var order in rangeOrderInfos)
                {
                    var first = dataNameGroup.First();
                    View_ServiceBillResultOrderInfo billResultOrderInfo = GetResolveOrderInfo(first, order);
                    allBillResoveOrderInfos.Add(billResultOrderInfo);
                }
            }

            var billProviderId = noneNumberResults.First().ServiceProviderId;
            var billTypeCode = noneNumberResults.First().BillTypeCode;
            var mappings = await GetMappings(billProviderId, billTypeCode);
 
            var spreadExpenseIds = allSpreadLogicDetails.Select(i => i.ExpenseId);
            var spreadExpenseMappings = mappings.Where(i => spreadExpenseIds.Contains(i.ExpenseId));
            var spreadExpense = await _expenseRepository.QueryAsync(i => i.Id.In(spreadExpenseIds));

            var dataNameResultGroups = allBillResoveOrderInfos.GroupBy(i =>  i.BillDataName );
            foreach (var g in dataNameResultGroups)
            {
                var first = g.First();
                var currency = first.BillCurrency;
                var rate = sysRates?.FirstOrDefault(i => i.Currency == currency && (i.StartDate <= first.BillTime && i.EndDate >= first.BillTime));


                string currentBillDataName = g.Key;
                var dataNameMappings = spreadExpenseMappings.Where(m => m.BillDataName == currentBillDataName);
                if (dataNameMappings?.Any() != true)
                {
                    throw new Exception($"账单费用名称【{currentBillDataName}】不存在费用映射");
                }
                else if (dataNameMappings.Count() > 1)
                {
                    throw new Exception($"账单费用名称【{currentBillDataName}】存在多条费用映射");
                }
                var mapping = spreadExpenseMappings.First();
                var expendSpreadDetails = allSpreadLogicDetails.Where(i => i.ExpenseId == mapping.ExpenseId);
                if (expendSpreadDetails?.Any() != true)
                {
                    throw new Exception($"映射的费用不存在分摊逻辑,费用ID:{ mapping.ExpenseId}");
                }
                else if (dataNameMappings.Count() > 1)
                {
                    throw new Exception($"映射的费用存在多个分摊逻辑,费用ID:{ mapping.ExpenseId}");
                }
                var expendSpreadDetail = expendSpreadDetails.First();
                var expense = spreadExpense.First(i => i.Id == expendSpreadDetail.ExpenseId);

                var groupData = g.ToList();
                await SpreadExpenseByLadingBill(groupData, expendSpreadDetail, expense, spreadApplyId,rate);
            }

        }

        private View_ServiceBillResultOrderInfo GetResolveOrderInfo(Finance_ServiceBillResolveResult first, View_OrderRangeOrderInfo order)
        {
            var info = new View_ServiceBillResultOrderInfo
            {
                BillCurrency = first.Currency,
                BillData = first.Data,
                BillDataName = first.BillDataName,
                BillName = first.BillName,
                BillNumber = first.Number,
                BillTime = first.BillTime,
                BillTypeCode = first.BillTypeCode,
                BoxNumber = order.BoxNumber,
                CustomerId = order.CustomerId,
                CustomerName = order.CustomerName,
                Height = order.Height,
                LadingBillNumber = order.LadingBillNumber,
                Length = order.Length,
                NumberType = first.NumberType,
                OPWeight = order.OPWeight,
                OrderChannelUniqueId = order.OrderChannelUniqueId,
                OrderTime = order.OrderTime,
                ReceiverCity = order.ReceiverCity,
                ReceiverCountryCode = order.ReceiverCountryCode,
                ReceiverCountryName = order.ReceiverCountryName,
                ReceiverState = order.ReceiverState,
                ReceiverStreet = order.ReceiverStreet,
                ReceiverZipCode = order.ReceiverZipCode,
                SystemNo = order.SystemNo,
                TrackingNumber = order.TrackingNumber,
                WeightRate = order.WeightRate,
                Width = order.Width
            };
            return info;
        }

        /// <summary>
        /// 提单形式的账单数据，通过提单去关联摊分逻辑，然后通过摊分逻辑，关联所有费用和映射
        /// </summary>
        /// <param name="ladingBillNumberResults"></param>
        /// <returns></returns>
        private async Task SpreadServiceBillByLadingBill(IEnumerable<View_ServiceBillResultOrderInfo> ladingBillNumberResults, int spreadApplyId)
        {
            var currencys = ladingBillNumberResults.Select(i => i.BillCurrency).Distinct();
            var sysRates = await _rateService.GetRates(currencys);

            var ladingBillResultGroups = ladingBillNumberResults.GroupBy(i => i.LadingBillNumber);
            var allLadingBillNumbers = ladingBillResultGroups.Select(i => i.Key).Where(i => !string.IsNullOrWhiteSpace(i));
            if (allLadingBillNumbers?.Any() != true)
            {
                throw new Exception("数据无提单号");
            }
            var allLadingBillInfos = await _ladingBillRepository.QueryByLadingBillNumbers(allLadingBillNumbers);
            var allSpreadLogicIds = allLadingBillInfos?.Where(i => i.SpreadlogicId != null && i.SpreadlogicId.Value > 0).Select(i => i.SpreadlogicId.Value);
            if (allSpreadLogicIds?.Any() != true)
            {
                throw new Exception("该账单的提单均无对应的均摊逻辑");
            }
            var allSpreadLogicDetails = await _SpreadlogicDetailRepository.QueryAsync(i => i.SpreadlogicId.In(allSpreadLogicIds));
            var allExpenseIds = allSpreadLogicDetails.Select(i => i.ExpenseId);

            string billTypeCode = ladingBillNumberResults.First().BillTypeCode;
            int serviceProviderId = ladingBillNumberResults.First().ServiceProviderId;
            var allMappings = await GetMappings(serviceProviderId, billTypeCode); 
            var allExpenses = await _expenseRepository.QueryAsync(e => e.Id.In(allExpenseIds));


            foreach (var ladingBillGroup in ladingBillResultGroups)
            {
                var billDataResult = ladingBillGroup.ToList();
                //按单个费用进行处理
                var billDataNameGroups = billDataResult.GroupBy(i => i.BillDataName);

                foreach (var dataNameGroup in billDataNameGroups)
                {
                    var first = ladingBillGroup.First();
                    var currency = first.BillCurrency;
                    var rate = sysRates?.FirstOrDefault(i => i.Currency == currency && (i.StartDate <= first.BillTime && i.EndDate >= first.BillTime));

                    string currentBillDataName = dataNameGroup.Key;
                    var mappings = allMappings?.Where(m => m.BillDataName == currentBillDataName);
                    if (mappings?.Any() != true)
                        throw new Exception($"找不到账单费用【{currentBillDataName}】与渠道均摊的费用之间的映射");
                    //实际上一个账单费用名称可能会对应多个系统费用，但目前只限制为一个
                    else if (mappings.Count() > 1)
                        throw new Exception($"账单费用【{currentBillDataName}】与渠道均摊的费用存在多条映射,系统费用ID:{string.Join(",", mappings.Select(m => m.ExpenseId))}");

                    var mapping = mappings.First();
                    var expense = allExpenses.First(e => e.Id == mapping.ExpenseId);
                    var expenseSpreadLogics = allSpreadLogicDetails.Where(s => s.ExpenseId == expense.Id);
                    if (expenseSpreadLogics?.Any() != true)
                        throw new Exception($"系统费用不存在摊分逻辑，费用ID:{expense.Id}");
                    if (expenseSpreadLogics.Count() > 1)
                        throw new Exception($"系统费用存在多条均摊方式");

                    var expendSpreadDetail = expenseSpreadLogics.First();

                    var groupData = dataNameGroup.ToList();

                    await SpreadExpenseByLadingBill(groupData, expendSpreadDetail, expense, spreadApplyId, rate);
                }


            }
        }

        private async Task<bool> SpreadExpenseByLadingBill(List<View_ServiceBillResultOrderInfo> ladingBillOrderNumberResults, Finance_SpreadlogicDetail spreadLogicDetail, Compute_Expense expense, int spreadApplyId, Sys_Rate rate)
        {

            var firstData = ladingBillOrderNumberResults.First();
            var billTypeCode = firstData.BillTypeCode;

            List<Compute_ExpenseAmount> spreadExpenseAmounts = new List<Compute_ExpenseAmount>();
            List<Logistisc_Order> orders = new List<Logistisc_Order>();

            var totalWeight = ladingBillOrderNumberResults.Sum(i => i.OPWeight);
            foreach (var r in ladingBillOrderNumberResults)
            {
                Logistisc_Order order = r;
                r.WeightRate = r.OPWeight / (decimal)totalWeight;
                orders.Add(order);
            }
            if (firstData.BillTime == null)
                throw new Exception("账单解析数据中的账单时间不能为空");


            string settlementCurrency = "CNY";
            decimal settlementAmount = firstData.BillData;
            decimal settlementRate = 1;
            if (rate != null)
            {
                settlementCurrency = rate.Currency;
                settlementAmount = firstData.BillData * rate.ToCNY;
                settlementRate = rate.ToCNY;
            }

            ComputeAmountResult amountResult = CreateComputeAmountResult(firstData, expense, settlementCurrency, settlementRate, settlementAmount);
            ApportionComputeRecordInput input = new ApportionComputeRecordInput
            {
                orders = orders,
                amountResult = amountResult,
                CostType = CostType.账单成本,
                detail = spreadLogicDetail,
                //newBillTypeCode = billTypeCode,
                //oldBillTypeCode = billTypeCode
            };

            var expenseAmounts = _orderCostService.ApportionComputeRecord(input);
            if (expenseAmounts?.Any() == true)
            {
                spreadExpenseAmounts.AddRange(expenseAmounts);
            }
            if (spreadExpenseAmounts.Count > 0)
            {
                spreadExpenseAmounts.ForEach(i =>
                {
                    i.ServiceBillSpreadApplyId = spreadApplyId;
                });

                var insertOk = await _orderCostService.InsertApportionComputeRecord(spreadExpenseAmounts);
                return insertOk;
            }
            else return false;
        }

        private async Task SpreadServiceBillByOrderNumber(IEnumerable<View_ServiceBillResultOrderInfo> billDataResults, int spreadApplyId)
        {
            var billTypeCode = billDataResults.First().BillTypeCode;
            var serviceProviderId = billDataResults.First().ServiceProviderId;

            var allMappings = await GetMappings(serviceProviderId, billTypeCode);

            var allExpenseIds = allMappings.Select(i => i.ExpenseId);

            var allExpenses = await _expenseRepository.QueryAsync(e => e.Id.In(allExpenseIds));
            if (allExpenses?.Any() != true)
            {
                throw new Exception("找不到系统费用信息");
            }

            var orderNumberDataGroups = billDataResults.GroupBy(i => i.BillDataName);
            foreach (var expenseGroup in orderNumberDataGroups)
            {
                string currentBillDataName = expenseGroup.Key;
                var groupBillDataResults = expenseGroup.ToList();

                var mappings = allMappings?.Where(m => m.BillDataName == currentBillDataName && allExpenseIds.Contains(m.ExpenseId));
                if (mappings?.Any() != true)
                    throw new Exception($"找不到账单费用【{currentBillDataName}】与渠道均摊的费用之间的映射");
                else if (mappings.Count() > 1)
                    throw new Exception($"账单费用【{currentBillDataName}】与渠道均摊的费用存在多条映射,系统费用ID:{string.Join(",", mappings.Select(m => m.ExpenseId))}");

                var mapping = mappings.First();
                var expense = allExpenses.First(e => e.Id == mapping.ExpenseId);

                await SpreadExpenseByOrderNumber(groupBillDataResults, expense, spreadApplyId);

            }

        }


        private void ValidateParam(SpreadServiceBillParam param)
        {
            if (param == null)
                throw new Exception("参数不能为空");
            if (string.IsNullOrWhiteSpace(param.BillName))
                throw new Exception("账单名称不能为空");
            if (param.SpreadLogicId <= 0)
                throw new Exception("均摊方案ID无效");
        }

        ///按订单号均摊一种费用
        private async Task<bool> SpreadExpenseByOrderNumber(IEnumerable<View_ServiceBillResultOrderInfo> orderNumberResults, Compute_Expense expense, int spreadApplyId)
        {
            var billTypeCode = orderNumberResults.First().BillTypeCode;


            var currencys = orderNumberResults.Select(i => i.BillCurrency).Distinct();
            var sysRates = await _rateService.GetRates(currencys);

            List<Compute_ExpenseAmount> spreadExpenseAmounts = new List<Compute_ExpenseAmount>();
            foreach (var r in orderNumberResults)
            {
                Logistisc_Order order = r;
                List<Logistisc_Order> orders = new List<Logistisc_Order> { order };

                if (r.BillTime == null)
                    throw new Exception("账单解析数据中的账单时间不能为空");


                var sysRate = sysRates?.FirstOrDefault(rate => rate.Currency == r.BillCurrency && (rate.StartDate <= r.BillTime.Value && rate.EndDate >= r.BillTime.Value));
                decimal settlementRate = sysRate != null ? sysRate.ToCNY : 1;
                string settlementCurrency = sysRate != null ? "CNY" : r.BillCurrency;
                var amt = new Compute_ExpenseAmount()
                {
                    Number = r.SystemNo,
                    ExpenseId = expense.Id,
                    ExpenseDetailId = 0,//实际账单不用关联费用明细
                    VersionId = 0,//实际账单不用关联版本
                    ExpenseItemId = 0,//expenseItemId,
                    TransactionTime = r.BillTime.Value,
                    Currency = r.BillCurrency,
                    AmountinOriginalCurrency = r.BillData,
                    SettlementRate = settlementRate,
                    SettlementCurrency = r.BillCurrency,
                    SettlementAmount = r.BillData,
                    FeeDescription = expense.Name,
                    BillTypeCode = billTypeCode,
                    CostType = CostType.账单成本,
                    CreateTime = DateTime.Now,
                };
                spreadExpenseAmounts.Add(amt);
            }



            if (spreadExpenseAmounts?.Any() == true)
            {
                spreadExpenseAmounts.ForEach(i =>
                {
                    i.ServiceBillSpreadApplyId = spreadApplyId;
                });
                var insertOk = await _orderCostService.InsertApportionComputeRecord(spreadExpenseAmounts);
                return insertOk;
            }
            else return false;

        }

        private ComputeAmountResult CreateComputeAmountResult(View_ServiceBillResultOrderInfo r, Compute_Expense expense, string settlementCurrency, decimal settlementRate, decimal settlementAmount)
        {
            ComputeAmountResult result = new ComputeAmountResult
            {
                Amount = settlementAmount,
                AmountinOriginalCurrency = r.BillData,
                BillCurrency = r.BillCurrency,
                BillNumber = r.SystemNo,
                Currency = settlementCurrency,
                DetailId = expense.Id,
                ExpenseName = expense.Name,
                ExpenseVersion = r.BillTime.Value,
                SettlementRate = settlementRate,
                VersionId = 0,
                ExpenseId = expense.Id,
                ExpenseItemId = 0,
            };
            return result;
        }

        public Task<bool> IsServiceBillOrderRangeSpread(string billName)
        {
            var existsNonNumberResult = _ServiceBillResolveResultRepository.IsExistsAsync(i => i.BillName == billName && i.NumberType == NumberType.无);
            return existsNonNumberResult;
        }
    }
}
